/*---------------------------------------------------------------------------*/
/* File: FrSegList.c                          Last update:    Apr 18, 2006   */
/*                                                                           */
/* Copyright (C) 2006, B. Mours,                                             */
/* For the licensing terms see the LICENSE file.                             */
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "FrSegListTool.h"

/*----------------------- To add:
Add a quite option; 
Tools:
Fabriquer un vecteur a partir d'un segement
Mettre dans FrCOpy l'option segement list pour choisir les donnes
-----------------*/
void Help()
{
 printf(" \n" 
     " This program provide a segement list using some selection criteria  \n"
     "-------------------------------------------------------------------- \n"
     " Syntax is: FrDump  -i <input file(s)>  (one or several file names   \n"
     "                    -f <first frame: (GPS time)>                     \n" 
     "                    -l <last  frame: (GPS time)> or length in second \n"
     "                    -min <list of tag channels>  <value>             \n" 
     "                    -max <debug level>                               \n"
     "                    -c <channel name>                                \n"
     "                    -h (to get this help)                            \n"
     "                    -s <segment list file name>                      \n" 
     "-------------------------------------------------------------------- \n");
}

struct Parameters{
  char *fileName;                   /* input file name                  */
  char *channelName;                    /* output tag                       */
  double ftime;                 /* time for the first frame         */
  double ltime;                 /* time for the last frame          */
  double min;                  /* minimum value                     */
  double max;                   /* maximum value                     */
  int nSegments;
  FRBOOL status;
  double tTot;
  FrSegList *segList; /* root pointer for the input segments */
  FrSegList *outList; /* result for the segment list */
  char      *outListName;
};

enum Type {INPUT, CHANNEL, FIRST, LAST, MIN, MAX, SEGMENT, OUT};

void ReadParameters(int argc, char **argv);
void StrCat(char **old, char *more);

struct Parameters Par;

/*------------------------------------------------------------SetStatus------*/
void SetStatus(FRBOOL status, double gtime)
/*---------------------------------------------------------------------------*/
{static double start;
 
 if(status == FR_YES && Par.status == FR_NO)
   {start = gtime;}
 else if(status == FR_NO && Par.status == FR_YES)
   {FrSegListAdd(Par.outList, start, gtime);}
      
 Par.status = status;

 return;}

/*--------------------------------------------------------- Main ------------*/
void ReadFile(FrFile* iFile, double length)
/*---------------------------------------------------------------------------*/
{FRBOOL status;
  double value, tStart, lastTime, min, max;
 int i,j, nVectors, lastPrintTime, curPrintTime;
 FrVect *frameInfo, *vect;

 lastTime = 0;
 Par.status = FR_NO;
 Par.nSegments = 0;
 nVectors = 0;
 max = -1.e37;
 min = 1.e37;
 Par.tTot = 0;

 /*-------------------------------------*/

 frameInfo = FrFileIGetFrameInfo(iFile, Par.ftime, length);
 if(frameInfo == NULL) 
   {printf(" No frames in the time range\n");
    return;}
 printf("%9d frames in this time window; \n", (int) frameInfo->nData);

 /*------ keep only frame inside the time range -----*/

 if(Par.segList != NULL)
    {j = 0;
     for(i=0; i<frameInfo->nData; i++)
       {frameInfo      ->dataD[j] = frameInfo      ->dataD[i];
	frameInfo->next->dataD[j] = frameInfo->next->dataD[i];
        if(FrSegListFindFirst(Par.segList,frameInfo->dataD[i], 
                       frameInfo->next->dataD[i]) >=0 ) j++;}
     printf("%9d frames after applying the segment list selection\n", j);
     frameInfo->nData = j;
     frameInfo->next->nData = j;
     if(frameInfo->nData == 0) exit(0);}

 /*-------------- start reading ----------------------*/

 printf(" Start reading...\n");
 lastPrintTime = time(NULL);

 for(i=0; i < frameInfo->nData; i++)
   {curPrintTime = time(NULL);
    if(lastPrintTime != curPrintTime)
      {lastPrintTime  = curPrintTime;
       printf(" %9d segments found after reading %9d frames (%4.1f%%)\r",
	  (int) Par.outList->tStart->nData,i, (100.*i)/frameInfo->nData);
       fflush(stdout);}
    tStart = frameInfo->dataD[i];
    length = frameInfo->next->dataD[i];
    if(tStart < Par.ftime) tStart = Par.ftime;
    if(tStart+length > Par.ltime) length = Par.ltime-tStart;
  
    vect = FrFileIGetVect(iFile, Par.channelName, tStart, length);
    if(vect == NULL) continue;

    nVectors++;
    if(vect->GTime - lastTime > .5*vect->dx[0]) SetStatus(FR_NO, lastTime);

    for(j=0; j<vect->nData; j++)
      {value = FrVectGetValueI(vect, j);
	if(value > max) max = value;
	if(value < min) min = value;
	if(value <= Par.max && value >= Par.min) 
             status = FR_YES;
	else status = FR_NO;
	SetStatus(status, vect->GTime+j*vect->dx[0]);}
    lastTime = vect->GTime+vect->nData*vect->dx[0];

    FrVectFree(vect);}

 SetStatus(FR_NO, lastTime);

 printf("   %12d vectors processed: min=%g max=%g                       \n\n",
                             nVectors, min, max);
 FrSegListDump(Par.outList, stdout, 2);
 Par.outList = FrSegListIntersect(Par.outList,Par.segList);
 FrSegListDump(Par.outList, stdout, 2);

 return;}

/*------------------------------------------------------ SetParameters ------*/
void ReadParameters(int argc, char **argv)
/*---------------------------------------------------------------------------*/
{int type, i;
  FrSegList *seg;

 if (argc == 1)
    {Help();
     exit(0);}

           /*-------------------- Default values ----------------------------*/

 Par.fileName    = NULL;
 Par.channelName = NULL;
 Par.ftime  = 0;
 Par.ltime  = 1.e12;
 Par.min    = -1.e37;
 Par.max    =  1.e37;
 Par.segList = NULL;
 Par.outList = FrSegListNew("FrSegList");
 Par.outListName = NULL;

           /*------------------- Loop over the parameters -------------------*/
 printf("-----------Read parameters--------------\n");

 type = 0;

 for (i=1; i<argc; i++)
    {if     (strcmp(argv[i],"-h") == 0) 
       {Help(); 
        exit(0);}
    else if (strcmp(argv[i],"-i") == 0) {type = INPUT;}
    else if (strcmp(argv[i],"-c") == 0) {type = CHANNEL;}
    else if (strcmp(argv[i],"-f") == 0) {type = FIRST;}
    else if (strcmp(argv[i],"-l") == 0) {type = LAST;}
    else if (strcmp(argv[i],"-min") == 0) {type = MIN;}
    else if (strcmp(argv[i],"-max") == 0) {type = MAX;}
    else if (strcmp(argv[i],"-s") == 0) {type = SEGMENT;}
    else if (strcmp(argv[i],"-o") == 0) {type = OUT;}
    else
      {if(type == INPUT)   {StrCat(&(Par.fileName),argv[i]);} 
       if(type == CHANNEL) {StrCat(&(Par.channelName),argv[i]);} 
       if(type == OUT)     {StrCat(&(Par.outListName),argv[i]);} 
       if(type == SEGMENT) 
	 {printf("  Read segment list: %s\n",argv[i]);
          seg = FrSegListRead(argv[i]);
          if(seg == NULL)
	    {fprintf(stderr," COuld not read segment list:%s\n",argv[i]);
             exit(0);}
	  seg->next = Par.segList;
	  Par.segList = seg;} 
       if(type == FIRST)   {Par.ftime = atof(argv[i]);}
       if(type == LAST)    {Par.ltime = atof(argv[i]);}
       if(type == MIN)     {Par.min   = atof(argv[i]);} 
       if(type == MAX)     {Par.max   = atof(argv[i]);}  
      }}
           /*-------------------------- Dump parameters ---------------------*/

 printf("  Input  Files     : %s\n", Par.fileName);
 printf("  Channel          : %g<= %s <= %g\n", 
                         Par.min, Par.channelName, Par.max);

           /*------------------ Check start/stop parameters------------------*/

 if(Par.fileName != NULL && Par.channelName == NULL ) 
   {fprintf(stderr," Please provide a channel name\n");
    exit(0);}
 if(Par.fileName == NULL && Par.channelName != NULL ) 
   {fprintf(stderr," Please provide a file name\n");
    exit(0);}

 return;
}
/*----------------------------------------------------------------- StrCat --*/
void StrCat(char **old, char *more)
/*---------------------------------------------------------------------------*/
{int lenMore;
 char *new;

 lenMore = strlen(more);

 if(*old == NULL)
   {*old =  malloc(strlen(more) + 1);
    strcpy(*old, more); 
    return;}

 new = malloc( strlen(*old) + lenMore + 2);
 sprintf(new,"%s %s",*old, more);
 free(*old);
 *old = new;

 return;}

/*--------------------------------------------------------- Main ------------*/
int main(int argc, char **argv)
/*---------------------------------------------------------------------------*/
{FrFile *iFile;         
 double length, lastTime, min, max;
 int nVectors;
 FrSegList *seg, *intersect;

           /*----------------- Read input arguments -------------------------*/

 ReadParameters(argc, argv);

           /*---------------------------- Open the input file(s) ------------*/

 if(Par.fileName != NULL)
   {iFile = FrFileINew(Par.fileName);
    if(iFile == NULL)
      {fprintf(stderr,"Cannot open input file %s\n %s",
                    Par.fileName, FrErrorGetHistory()); 
	return(0);}
     if(Par.ftime == 0) Par.ftime = FrFileITStart(iFile);}

           /*----------------Start the Main loop ----------------------------*/

 lastTime = 0;
 Par.status = FR_NO;
 Par.nSegments = 0;
 nVectors = 0;
 max = -1.e37;
 min = 1.e37;
 Par.tTot = 0;

 if(Par.ltime < Par.ftime) Par.ltime += Par.ftime -1.e-6;
 length = Par.ltime-Par.ftime;

 printf("  Start time       : %.1f\n", Par.ftime);
 printf("  Duration         : %.1f\n", length);

 /*------------- read the input segment list and get the intersection-----*/

 if(Par.segList != NULL)
   {printf("----------------------------------------\n Segments info:\n");
    FrSegListShrink(Par.segList, Par.ftime, Par.ltime);
    FrSegListDump(Par.segList, stdout, 2);
    for(seg = Par.segList->next; seg != NULL; seg = seg->next)
     {FrSegListShrink(seg, Par.ftime, Par.ltime);
       FrSegListDump(seg, stdout, 2);
      intersect = FrSegListIntersect(Par.segList, seg);
      Par.segList->next = NULL;
      FrSegListFree(Par.segList);
      Par.segList = intersect;}

    printf(" The intersection of the input segment list is:\n");
    FrSegListDump(Par.segList, stdout, 2);

    if(Par.segList->tStart->nData == 0) return(0);
    Par.ftime = Par.segList->tStart->dataD[0];
    length    = Par.segList->tEnd->dataD[Par.segList->tStart->nData-1]-Par.ftime;
    printf(" Updated Start time:%.1f and duration: %.1f\n",Par.ftime, length);}

 /*------------------------read file if needed---*/

 if(Par.fileName != NULL) ReadFile(iFile, length);
 else Par.outList = Par.segList;

 if(Par.outListName != NULL) FrSegListPrint(Par.outList, Par.outListName);

 exit(0);
}
